@@ -1,30 +1,33 @@
 #!/usr/bin/python3
 import apt
+import argparse
 import datetime
-import gettext
 import gi
-import locale
+import json
 import os
+import platform
 import setproctitle
 import shutil
 import subprocess
+import sys
+import threading
+import xapp.SettingsWidgets as Xs
+import xapp.threading as xt
+import xapp.util
 
 gi.require_version("Gtk", "3.0")
 gi.require_version('GtkSource', '3.0')
 gi.require_version('XApp', '1.0')
-from gi.repository import Gtk, Gdk, GtkSource, Gio, XApp
+from gi.repository import Gtk, Gdk, GtkSource, Gio
 
-from common import _async, idle, InfoReportContainer, DATA_DIR, INFO_DIR
+from common import InfoReportContainer, DATA_DIR, INFO_DIR, prefix_version, read_dmi, clean_brand
+from bios import BIOSListWidget
+from pci import PCIListWidget
+from usb import USBListWidget
+from gpu import GPUListWidget
 
 setproctitle.setproctitle("mintreport")
-
-# i18n
-APP = 'mintreport'
-LOCALE_DIR = "/usr/share/locale"
-locale.bindtextdomain(APP, LOCALE_DIR)
-gettext.bindtextdomain(APP, LOCALE_DIR)
-gettext.textdomain(APP)
-_ = gettext.gettext
+_ = xapp.util.l10n("mintreport")
 
 TMP_DIR = "/tmp/mintreport"
 TMP_INFO_DIR = os.path.join(TMP_DIR, "reports")
@@ -50,17 +53,110 @@ class MyApplication(Gtk.Application):
     def __init__(self, application_id, flags):
         Gtk.Application.__init__(self, application_id=application_id, flags=flags)
         self.connect("activate", self.activate)
+        self.connect("command-line", self.on_command_line)
+        self.report_window = None
+        self.page = None
+
+    def on_command_line(self, app, command_line):
+        # Parse fresh args each time
+        parser = argparse.ArgumentParser(add_help=False)
+        group = parser.add_mutually_exclusive_group()
+        group.add_argument("--reports", action="store_const", dest="page", const="reports")
+        group.add_argument("--crashes", action="store_const", dest="page", const="crashes")
+        group.add_argument("--usb", action="store_const", dest="page", const="usb")
+        group.add_argument("--info", action="store_const", dest="page", const="info")
+        group.add_argument("--bios", action="store_const", dest="page", const="bios")
+        group.add_argument("--pci", action="store_const", dest="page", const="pci")
+        group.add_argument("--gpu", action="store_const", dest="page", const="gpu")
+        argv = command_line.get_arguments()[1:]
+        args, _ = parser.parse_known_args(argv)
+        if args.page is None:
+            args.page = "info"
+        self.page = args.page
+        self.activate(app)
+        return 0
 
     def activate(self, application):
-        windows = self.get_windows()
-        if (len(windows) > 0):
-            window = windows[0]
-            window.present()
-            window.show_all()
-        else:
-            window = MintReportWindow(self)
-            self.add_window(window.window)
-            window.window.show_all()
+        if self.report_window is None:
+            self.report_window = MintReportWindow(self)
+            self.add_window(self.report_window.window)
+        window = self.report_window.window
+        window = self.get_windows()[0]
+        window.present()
+        window.show_all()
+        self.report_window.show_page(self.page)
+
+def kill_process(process):
+    process.kill()
+
+def get_process_output(command):
+    timeout = 2.0  # Timeout for any subprocess before aborting it
+
+    lines = []
+    p = subprocess.Popen(command, stdout=subprocess.PIPE)
+    timer = threading.Timer(timeout, kill_process, [p])
+    timer.start()
+    while True:
+        line = p.stdout.readline()
+        if not line:
+            break
+        if line != '':
+            lines.append(line.decode('utf-8'))
+    timer.cancel()
+    return lines
+
+
+def get_graphic_cards():
+    cards = {}
+    count = 0
+    envpath = os.environ["PATH"]
+    os.environ["PATH"] = envpath + ":/usr/local/sbin:/usr/sbin:/sbin"
+    for card in get_process_output("lspci"):
+        for prefix in ["VGA compatible controller:", "3D controller:", "Display controller:"]:
+            if prefix in card:
+                cardName = card.split(prefix)[1].split("(rev")[0].strip()
+                cards[count] = cardName
+                count += 1
+    os.environ["PATH"] = envpath
+    return cards
+
+
+def get_disk_size():
+    disksize = 0
+    try:
+        out = get_process_output(("lsblk", "--json", "--output", "size", "--bytes", "--nodeps"))
+        jsonobj = json.loads(''.join(out))
+    except Exception:
+        return _("Unknown size"), False
+
+    for blk in jsonobj['blockdevices']:
+        disksize += int(blk['size'])
+
+    return disksize, (len(jsonobj['blockdevices']) > 1)
+
+
+def get_proc_infos():
+    # For some platforms, 'model name' will no longer take effect.
+    # We can try our best to detect it, but if all attempts failed just leave it to be "Unknown".
+    # Source: https://github.com/dylanaraps/neofetch/blob/6dd85d67fc0d4ede9248f2df31b2cd554cca6c2f/neofetch#L2163
+    cpudetect = ("model name", "Hardware", "Processor", "cpu model", "chip type", "cpu type")
+    infos = [
+        ("/proc/cpuinfo", [("cpu_name", cpudetect), ("cpu_siblings", ("siblings",)), ("cpu_cores", ("cpu cores",))]),
+        ("/proc/meminfo", [("mem_total", ("MemTotal",))])
+    ]
+    result = {}
+    for (proc, pairs) in infos:
+        for line in get_process_output(("cat", proc)):
+            for (key, start) in pairs:
+                for item in start:
+                    if line.startswith(item):
+                        result[key] = line.split(':', 1)[1].strip()
+                        break
+    if "cpu_name" not in result:
+        result["cpu_name"] = _("Unknown CPU")
+    if "mem_total" not in result:
+        result["mem_total"] = _("Unknown size")
+    return result
 
 class MintReportWindow():
 
@@ -74,24 +170,35 @@ class MintReportWindow():
         os.system("mkdir -p %s" % UNPACK_DIR)
         os.system("cp -R %s/* %s/" % (DATA_DIR, TMP_DIR))
 
+        # Create a CSS provider
+        css_provider = Gtk.CssProvider()
+        css_provider.load_from_data(b"""
+            .logo-box {
+                background-color: #86be43;
+                border-radius: 10px;
+                padding: 12px;
+            }
+        """)
+        screen = Gdk.Screen.get_default()
+        Gtk.StyleContext.add_provider_for_screen(
+            screen,
+            css_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
+
         self._cache = None
         # Set the Glade file
         gladefile = "/usr/share/linuxmint/mintreport/mintreport.ui"
         self.builder = Gtk.Builder()
-        self.builder.set_translation_domain(APP)
+        self.builder.set_translation_domain("mintreport")
         self.builder.add_from_file(gladefile)
         self.window = self.builder.get_object("main_window")
-        self.window.set_title(_("System Reports"))
+        self.window.set_title(_("System Information"))
         self.window.set_icon_name("mintreport")
 
-        self.stack = self.builder.get_object("crash_stack")
+        self.stack = self.builder.get_object("main_stack")
         self.spinner = self.builder.get_object("crash_spinner")
 
-        # Fill in the sysinfo pane
-        self.load_sysinfo()
-        self.builder.get_object("button_sysinfo_copy").connect("clicked", self.copy_sysinfo)
-        self.builder.get_object("button_sysinfo_upload").connect("clicked", self.upload_sysinfo)
-
         # the crashes treeview
         self.treeview_crashes = self.builder.get_object("treeview_crashes")
 
@@ -124,6 +231,7 @@ class MintReportWindow():
         column = Gtk.TreeViewColumn('', cell_renderer)
         column.set_cell_data_func(cell_renderer, render_date)
         column.set_resizable(True)
+        cell_renderer.set_property("ypad", 6)
         self.treeview_crashes.append_column(column)
         column = Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=COL_CRASH_PID)
         column.set_sort_column_id(COL_CRASH_PID)
@@ -166,7 +274,9 @@ class MintReportWindow():
         column = Gtk.TreeViewColumn("", renderer, icon_name=COL_INFO_ICON)
         self.treeview_info.append_column(column)
 
-        column = Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=COL_INFO_TITLE)
+        renderer = Gtk.CellRendererText()
+        renderer.set_property("ypad", 6)
+        column = Gtk.TreeViewColumn("", renderer, text=COL_INFO_TITLE)
         column.set_sort_column_id(COL_INFO_TITLE)
         column.set_resizable(True)
         self.treeview_info.append_column(column)
@@ -183,7 +293,15 @@ class MintReportWindow():
         else:
             self.builder.get_object("crash_internal_stack").set_visible_child_name("page_error")
 
-        self.load_info()
+        page = Xs.SettingsPage()
+        page.set_spacing(24)
+        page.set_margin_left(0)
+        page.set_margin_right(0)
+        page.set_margin_top(0)
+        page.set_margin_bottom(0)
+        self.builder.get_object("box_info_widget").add(page)
+
+        self.sysinfo_section = page.add_section()
 
         accel_group = Gtk.AccelGroup()
         self.window.add_accel_group(accel_group)
@@ -202,7 +320,7 @@ class MintReportWindow():
         menu.append(Gtk.SeparatorMenuItem())
 
         item = Gtk.ImageMenuItem(label=_("Quit"))
-        image = Gtk.Image.new_from_icon_name("application-exit-symbolic", Gtk.IconSize.MENU)
+        image = Gtk.Image.new_from_icon_name("xsi-exit-symbolic", Gtk.IconSize.MENU)
         item.set_image(image)
         item.connect('activate', self.on_menu_quit)
         key, mod = Gtk.accelerator_parse("<Control>Q")
@@ -212,13 +330,124 @@ class MintReportWindow():
         menu.append(item)
 
         item = Gtk.ImageMenuItem()
-        item.set_image(Gtk.Image.new_from_icon_name("help-about-symbolic", Gtk.IconSize.MENU))
+        item.set_image(Gtk.Image.new_from_icon_name("xsi-help-about-symbolic", Gtk.IconSize.MENU))
         item.set_label(_("About"))
         item.connect("activate", self.open_about)
         menu.append(item)
 
         menu.show_all()
 
+        # Info page
+        procInfos = get_proc_infos()
+        infos = []
+
+         # Product
+        version = prefix_version(read_dmi('product_version'))
+        vendor = clean_brand(read_dmi('sys_vendor'))
+        sku = read_dmi('product_sku')
+        product = f"{vendor} {read_dmi('product_name')} {version} {sku}"
+        product = " ".join(product.split()) # remove unecessary spaces
+        infos.append((_("Computer"), product))
+
+        try:
+            (memsize, memunit) = procInfos['mem_total'].split(" ")
+            memsize = float(memsize)
+        except ValueError:
+            memsize = procInfos['mem_total']
+            memunit = ""
+        processorName = procInfos['cpu_name'].replace("(R)", "\u00A9").replace("(TM)", "\u2122")
+        if 'cpu_cores' in procInfos:
+            processorName = processorName + " \u00D7 " + procInfos['cpu_cores']
+
+        with open("/etc/linuxmint/info") as f:
+            config = dict([line.strip().split("=") for line in f])
+        distribution = "Linux Mint"
+        if os.path.exists("/usr/share/doc/debian-system-adjustments/copyright"):
+            distribution = "LMDE"
+        release = config['RELEASE']
+        edition = config['EDITION'].replace('"', '')
+        architecture = "64-bit"
+        if platform.machine() not in ["x86_64", "aarch64"]:
+            architecture = "32-bit"
+        infos.append((_("Operating System"), f"{distribution} {release} - {edition} {architecture}"))
+        infos.append((_("Linux Kernel"), platform.release()))
+        infos.append((_("Processor"), processorName))
+        cards = get_graphic_cards()
+        for card in cards:
+            infos.append((_("Graphics Card"), cards[card]))
+        if memunit == "kB":
+            infos.append((_("Memory"), '%.1f %s' % ((float(memsize)/(1000**2)), _("GB"))))
+        else:
+            infos.append((_("Memory"), procInfos['mem_total']))
+
+        diskSize, multipleDisks = get_disk_size()
+        if multipleDisks:
+            diskText = _("Hard Drives")
+        else:
+            diskText = _("Hard Drive")
+        try:
+            infos.append((diskText, '%.1f %s' % ((diskSize / (1000**3)), _("GB"))))
+        except:
+            infos.append((diskText, diskSize))
+
+        desktop = os.getenv("XDG_CURRENT_DESKTOP")
+        if "cinnamon" in desktop.lower():
+            desktop = f"Cinnamon {os.getenv("CINNAMON_VERSION")}"
+        infos.append((_("Desktop Environment"), desktop))
+
+        display_server_name = "X11"
+
+        if os.getenv("XDG_SESSION_TYPE") == "wayland":
+            display_server_name = "Wayland"
+
+        infos.append((_("Display Server"), display_server_name))
+
+        for (key, value) in infos:
+            self.sysinfo_section.add_row(self.create_info_row(key, value))
+
+        self.inxi_info = ""
+        self.builder.get_object("button_sysinfo_copy").connect("clicked", self.copy_inxi_info)
+        self.builder.get_object("button_sysinfo_upload").connect("clicked", self.upload_inxi_info)
+
+
+        # USB page
+        self.usb_widget = USBListWidget()
+        self.builder.get_object("box_usb_widget").pack_start(self.usb_widget, True, True, 0)
+
+        # PCI page
+        self.pci_widget = PCIListWidget()
+        self.builder.get_object("box_pci_widget").pack_start(self.pci_widget, True, True, 0)
+
+        # BIOS page
+        self.bios_widget = BIOSListWidget()
+        self.builder.get_object("box_bios_widget").add(self.bios_widget)
+
+        self.gpu_widget = GPUListWidget()
+        self.builder.get_object("box_gpu_widget").add(self.gpu_widget)
+
+        self.load_inxi_info()
+        self.load_reports()
+        self.load_usb()
+        self.load_pci()
+        self.load_bios()
+        self.load_gpu()
+
+    def show_page(self, page_name):
+        page_name = f"page_{page_name}"
+        self.stack.set_visible_child_name(page_name)
+
+    def create_info_row(self, key, value):
+        widget = Xs.SettingsWidget()
+        widget.set_spacing(40)
+        labelKey = Gtk.Label.new(key)
+        widget.pack_start(labelKey, False, False, 0)
+        labelKey.get_style_context().add_class("dim-label")
+        labelValue = Gtk.Label.new(value)
+        labelValue.set_selectable(True)
+        labelValue.set_line_wrap(True)
+        widget.pack_end(labelValue, False, False, 0)
+        return widget
+
     @property
     def cache(self):
         """Cache data from apt"""
@@ -233,8 +462,8 @@ class MintReportWindow():
         dlg = Gtk.AboutDialog()
         dlg.set_transient_for(self.window)
         dlg.set_title(_("About"))
-        dlg.set_program_name("mintReport")
-        dlg.set_comments(_("System Reports"))
+        dlg.set_program_name("mintreport")
+        dlg.set_comments(_("System Information"))
         try:
             h = open('/usr/share/common-licenses/GPL', encoding="utf-8")
             s = h.readlines()
@@ -258,69 +487,65 @@ class MintReportWindow():
 
     def on_menu_restore(self, widget):
         self.settings.reset("ignored-reports")
-        self.load_info()
+        self.load_reports()
 
     def on_menu_refresh(self, widget):
-        self.load_info()
-        self.load_sysinfo()
+        self.load_reports()
         self.load_crashes()
 
     def on_menu_quit(self, widget):
         self.application.quit()
 
-    @_async
-    def load_sysinfo(self):
+    @xt.run_async
+    def load_inxi_info(self):
         try:
             sysinfo = subprocess.check_output("LANG=C inxi -Fxxxrzc0 --usb", shell=True).decode("utf-8", errors='replace')
-            self.add_sysinfo_to_textview(sysinfo)
+            self.inxi_info = sysinfo
+            self.enable_inxi_buttons()
             with open(TMP_INXI_FILE, "w") as f:
                 f.write(sysinfo)
         except Exception as e:
-            subprocess.Popen(['notify-send', '-i', 'dialog-error-symbolic', _("An error occurred while gathering the system information."), str(e)])
+            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-error-symbolic', _("An error occurred while gathering the system information."), str(e)])
             print (e)
 
-    @idle
-    def add_sysinfo_to_textview(self, text):
-        buff = Gtk.TextBuffer()
-        buff.set_text(text)
-        self.builder.get_object("textview_sysinfo").set_buffer(buff)
+    @xt.run_idle
+    def enable_inxi_buttons(self):
+        self.builder.get_object("button_sysinfo_copy").set_sensitive(True)
+        self.builder.get_object("button_sysinfo_upload").set_sensitive(True)
 
-    def copy_sysinfo(self, button):
+    def copy_inxi_info(self, button):
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        buff = self.builder.get_object("textview_sysinfo").get_buffer()
-        text = buff.get_text(buff.get_start_iter(), buff.get_end_iter(), False)
-        clipboard.set_text("[code]\n%s[/code]\n" % text, -1)
-        subprocess.Popen(['notify-send', '-i', 'dialog-information-symbolic', _("System information copied"), _("Your system information was copied into your clipboard so you can paste it on the forums.")])
+        clipboard.set_text("[code]\n%s[/code]\n" % self.inxi_info, -1)
+        subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("System information copied"), _("Your system information was copied into your clipboard so you can paste it on the forums.")])
 
-    def upload_sysinfo(self, button):
+    def upload_inxi_info(self, button):
         try:
             output = subprocess.check_output("pastebin %s" % TMP_INXI_FILE, shell=True).decode("UTF-8")
             link = output.rstrip('\x00').strip() # Remove ASCII null termination with \x00
             clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-            buff = self.builder.get_object("textview_sysinfo").get_buffer()
             clipboard.set_text(link, -1)
-            subprocess.Popen(['notify-send', '-i', 'dialog-information-symbolic', _("System information uploaded"), _("Your system information was uploaded to %s. This link was placed in your clipboard.") % link])
+            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("System information uploaded"), _("Your system information was uploaded to %s. This link was placed in your clipboard.") % link])
         except Exception as e:
-            subprocess.Popen(['notify-send', '-i', 'dialog-error-symbolic', _("An error occurred while uploading the system information"), _("Copy and paste the system information manually into a pastebin site like https://pastebin.com."), str(e)])
+            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-error-symbolic', _("An error occurred while uploading the system information"), _("Copy and paste the system information manually into a pastebin site like https://pastebin.com."), str(e)])
 
-    @idle
+    @xt.run_idle
     def add_report_to_treeview(self, report):
         iter = self.model_info.insert_before(None, None)
         self.model_info.set_value(iter, COL_INFO_ICON, report.instance.icon)
         self.model_info.set_value(iter, COL_INFO_TITLE, report.instance.title)
         self.model_info.set_value(iter, COL_INFO_REPORT, report)
 
-    @idle
+    @xt.run_idle
     def clear_info_treeview(self):
         self.model_info.clear()
         self.builder.get_object("info_box").hide()
 
-    @idle
+    @xt.run_idle
     def show_info_spinner(self):
         self.builder.get_object("box_info_stack").set_visible_child_name("spinner")
         self.builder.get_object("info_spinner").start()
 
-    @idle
+    @xt.run_idle
     def show_info_reports(self):
         if self.num_info_found > 0:
             self.builder.get_object("box_info_stack").set_visible_child_name("reports")
@@ -331,8 +556,8 @@ class MintReportWindow():
             self.builder.get_object("box_info_stack").set_visible_child_name("done")
         self.builder.get_object("info_spinner").stop()
 
-    @_async
-    def load_info(self):
+    @xt.run_async
+    def load_reports(self):
         self.loading = True
         self.clear_info_treeview()
         self.show_info_spinner()
@@ -354,6 +579,22 @@ class MintReportWindow():
         self.loading = False
         self.show_info_reports()
 
+    @xt.run_async
+    def load_usb(self):
+        self.usb_widget.load()
+
+    @xt.run_async
+    def load_pci(self):
+        self.pci_widget.load()
+
+    @xt.run_async
+    def load_bios(self):
+        self.bios_widget.load()
+
+    @xt.run_async
+    def load_gpu(self):
+        self.gpu_widget.load()
+
     def on_info_selected(self, selection):
         if self.loading:
             return
@@ -389,7 +630,7 @@ class MintReportWindow():
         self.window.set_sensitive(False)
         reload_requested = callback(data)
         if reload_requested:
-            self.load_info()
+            self.load_reports()
         self.window.set_sensitive(True)
 
     def on_ignore_button_clicked(self, button):
@@ -410,7 +651,7 @@ class MintReportWindow():
                         if self.num_info_found == 0:
                             self.show_info_reports()
 
-    @idle
+    @xt.run_idle
     def load_crashes(self):
         self.loading = True
         self.model_crashes.clear()
@@ -442,7 +683,7 @@ class MintReportWindow():
     def on_crash_selected(self, selection):
         if self.loading:
             return
-        self.stack.set_visible_child_name("page0")
+        self.builder.get_object("crash_stack").set_visible_child_name("page0")
         self.spinner.start()
         self.spinner.show()
         self.treeview_crashes.set_sensitive(False)
@@ -457,7 +698,7 @@ class MintReportWindow():
             report = model.get_value(iter, COL_CRASH_OBJECT)
             self.dump_crash_report(report)
 
-    @_async
+    @xt.run_async
     def dump_crash_report(self, report):
 
         os.chdir(UNPACK_DIR)
@@ -538,14 +779,14 @@ class MintReportWindow():
 
         self.on_unpack_crash_report_finished()
 
-    @idle
+    @xt.run_idle
     def on_unpack_crash_report_finished(self):
         self.treeview_crashes.set_sensitive(True)
         self.localfiles_button.set_sensitive(True)
         self.spinner.stop()
-        self.stack.set_visible_child_name("page1")
+        self.builder.get_object("crash_stack").set_visible_child_name("page1")
 
-    @idle
+    @xt.run_idle
     def show_stack_info(self, text):
         self.buffer.set_language(self.language_manager.get_language("gdb-log"))
         self.buffer.set_text(text)
@@ -566,5 +807,5 @@ class MintReportWindow():
         subprocess.call(['xdg-open', output])
 
 if __name__ == "__main__":
-    application = MyApplication("com.linuxmint.reports", Gio.ApplicationFlags.FLAGS_NONE)
-    application.run()
+    application = MyApplication("com.linuxmint.reports", Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
+    application.run(sys.argv)
